﻿// RMR ("aRMouR") Code Generator
// Copyright 2013 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of RMR Code Generator.

// RMR Code Generator is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// RMR Code Generator is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with RMR Code Generator (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace BenLincoln.RMRCodes
{
    public class RMRByte : SVGElement
    {
        protected int _ByteNumber;
        protected byte _Value;
        protected bool _Parity;

        #region Properties

        public int ByteNumber
        {
            get
            {
                return _ByteNumber;
            }
            set
            {
                _ByteNumber = value;
            }
        }

        public byte Value
        {
            get
            {
                return _Value;
            }
            set
            {
                _Value = value;
                CalculateParity();
            }
        }

        public bool Parity
        {
            get
            {
                return _Parity;
            }
            set
            {
                _Parity = value;
            }
        }

        #endregion

        public RMRByte()
            : base()
        {
        }

        public RMRByte(int xPosition, int yPosition)
            : base(xPosition, yPosition)
        {
        }

        protected override void Initialize()
        {
            base.Initialize();
            _ByteNumber = 0;
        }

        protected void CalculateParity()
        {
            byte[] bits = GetBits();

            byte par = (byte)((~(bits[0] ^ bits[1])) & 1);
            par = (byte)((~(par ^ bits[2])) & 1);
            par = (byte)((~(par ^ bits[3])) & 1);
            par = (byte)((~(par ^ bits[4])) & 1);
            par = (byte)((~(par ^ bits[5])) & 1);
            par = (byte)((~(par ^ bits[6])) & 1);
            par = (byte)((~(par ^ bits[7])) & 1);

            if (par > 0)
            {
                _Parity = true;
            }
            else
            {
                _Parity = false;
            }
        }

        public override string GetSVGCode()
        {
            string result = base.GetSVGCode();

            byte[] bits = GetBits();

            for (int i = 0; i < 8; i++)
            {
                if (bits[i] == 0)
                {
                    result = RemoveBit(result,  String.Format("bit_{0:000}", ((int)(Math.Pow(2, i)))));
                }
            }

            if (!Parity)
            {
                result = RemoveBit(result, "bit_parity");
            }

            string idString = String.Format("{0:0000000000000000}", ByteNumber);

            result = result.Replace("%BYTE_ID%", idString);

            return result;
        }

        protected string RemoveBit(string svgCode, string bitNumber)
        {
            string pattern = bitNumber;
            StringBuilder builder = new StringBuilder();
            Regex rex = new Regex(pattern, RegexOptions.IgnoreCase);
            string[] lines = svgCode.Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);
            foreach (string line in lines)
            {
                MatchCollection matches = rex.Matches(line);
                if (matches.Count == 0)
                {
                    builder.Append(line);
                    builder.Append(Environment.NewLine);
                }
            }
            return builder.ToString();
        }

        protected byte[] GetBits()
        {
            byte[] result = new byte[8];
            result[0] = (byte)(Value & 1);
            result[1] = (byte)((Value >> 1) & 1);
            result[2] = (byte)((Value >> 2) & 1);
            result[3] = (byte)((Value >> 3) & 1);
            result[4] = (byte)((Value >> 4) & 1);
            result[5] = (byte)((Value >> 5) & 1);
            result[6] = (byte)((Value >> 6) & 1);
            result[7] = (byte)((Value >> 7) & 1);

            return result;
        }
    }
}
