﻿// RMR ("aRMouR") Code Generator
// Copyright 2013 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of RMR Code Generator.

// RMR Code Generator is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// RMR Code Generator is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with RMR Code Generator (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Reflection;

namespace BenLincoln.RMRCodes
{
    public class SVGElement
    {
        protected const string _XPositionRegex = "x=\"[0-9\\.\\-]{1,64}\"";
        protected const string _YPositionRegex = "y=\"[0-9\\.\\-]{1,64}\"";

        protected string _EmbeddedResourceFile;

        protected int _BasePositionX;
        protected int _BasePositionY;
        protected int _Width;
        protected int _Height;

        #region Properties

        public int BasePositionX
        {
            get
            {
                return _BasePositionX;
            }
            set
            {
                _BasePositionX = value;
            }
        }

        public int BasePositionY
        {
            get
            {
                return _BasePositionY;
            }
            set
            {
                _BasePositionY = value;
            }
        }

        public int Width
        {
            get
            {
                return _Width;
            }
        }

        public int Height
        {
            get
            {
                return _Height;
            }
        }

        #endregion

        public SVGElement()
        {
            Initialize();
        }

        public SVGElement(int xPosition, int yPosition)
            :this()
        {
            BasePositionX = xPosition;
            BasePositionY = yPosition;
        }

        protected virtual void Initialize()
        {
            BasePositionX = 0;
            BasePositionY = 0;
        }

        public virtual string GetSVGCode()
        {
            string result = GetEmbeddedResourceFileAsString(_EmbeddedResourceFile);

            result = UpdateXPositions(result, BasePositionX);
            result = UpdateYPositions(result, BasePositionY);

            return result;
        }

        protected static string GetEmbeddedResourceFileAsString(string embeddedFileName)
        {
            String result = "";

            Assembly asm = Assembly.GetExecutingAssembly();
            Stream inStream = asm.GetManifestResourceStream(embeddedFileName);
            StreamReader inReader = new StreamReader(inStream);
            result = inReader.ReadToEnd();
            inReader.Close();
            inStream.Close();

            return result;
        }

        protected static string UpdatePositions(string regexString, string variableName, string inputSVGSCode, int basePosition)
        {
            string result = inputSVGSCode;
            Regex rex = new Regex(regexString, RegexOptions.IgnoreCase);
            MatchCollection matches = rex.Matches(inputSVGSCode);
            foreach (Match m in matches)
            {
                string originalText = m.Value;
                string baseValueString = "";
                double baseValue = 0.0d;
                int firstQuotePos = originalText.IndexOf('"');
                int lastQuotePos = originalText.LastIndexOf('"');
                StringBuilder replacementBuilder = new StringBuilder();
                replacementBuilder.Append(originalText.Substring(0, firstQuotePos + 1));
                baseValueString = originalText.Substring(firstQuotePos + 1, (lastQuotePos - (firstQuotePos + 1)));
                try
                {
                    baseValue = Double.Parse(baseValueString);
                }
                catch (Exception ex)
                {
                    throw new Exception("Couldn't parse a real number from the text '" + originalText + "'");
                }
                int newVal = (int)(basePosition + (int)baseValue);
                replacementBuilder.Append(newVal.ToString());
                replacementBuilder.Append("\"");
                result = result.Replace(originalText, replacementBuilder.ToString());
            }
            return result;
        }


        protected static string UpdateXPositions(string inputSVGSCode, int baseXPosition)
        {
            return UpdatePositions(_XPositionRegex, "x", inputSVGSCode, baseXPosition);
        }

        protected static string UpdateYPositions(string inputSVGSCode, int baseYPosition)
        {
            return UpdatePositions(_YPositionRegex, "y", inputSVGSCode, baseYPosition);
        }

    }
}
